<?php

namespace Drupal\page_cache_exclusion\StackMiddleware;

use Drupal\page_cache\StackMiddleware\PageCache;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\PageCache\RequestPolicyInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\PageCache\ResponsePolicyInterface;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Executes the page caching before the main kernel takes over the request.
 */
class PageCacheAlter extends PageCache
{

  /**
   * A config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;



  /**
   * {@inheritdoc}
   */
  public function __construct(HttpKernelInterface $http_kernel, CacheBackendInterface $cache, RequestPolicyInterface $request_policy, ResponsePolicyInterface $response_policy, ConfigFactoryInterface $config_factory) {
    parent::__construct($http_kernel, $cache, $request_policy, $response_policy);
    $this->config = $config_factory->get('page_cache_exclusion.settings');
  }


  /**
   * {@inheritdoc}
   */
  protected function set(Request $request, Response $response, $expire, array $tags) {
    $disableErrorResponse = $this->config->get('client_error_caching');
    if ($disableErrorResponse && $response->isClientError()) {
      return;
    }

    // Compare the lowercase path alias (if any) and internal path.
    $path = \Drupal::service('path.current')->getPath($request);
    // Do not trim a trailing slash if that is the complete path.
    $path = $path === '/' ? $path : rtrim($path, '/');
    $path_alias = mb_strtolower(\Drupal::service('path_alias.manager')->getAliasByPath($path));
    
    $pageExclusionList = $this->config->get('page_list');
    $matchedPageExclusion = \Drupal::service('path.matcher')->matchPath($path_alias, $pageExclusionList) || (($path != $path_alias) && \Drupal::service('path.matcher')->matchPath($path, $pageExclusionList));
    if($pageExclusionList && $matchedPageExclusion){
      return;
    }

    $pageQueryParametersList = $this->config->get('page_query_parameters_list');
    $queryParameters = $request->query->all();
    $matched = \Drupal::service('path.matcher')->matchPath($path_alias, $pageQueryParametersList) || (($path != $path_alias) && \Drupal::service('path.matcher')->matchPath($path, $pageQueryParametersList));
    if($matched && $queryParameters) {
      return;
    }

    $cid = $this->getCacheId($request);
    $this->cache->set($cid, $response, $expire, $tags);
  }
}
